#define ADC0						0
#define ADC1						1
#define ADC2						2
#define ADC3						3
#define ADC4						4
#define ADC5						5
#define ADC6						6
#define ADC7						7

// defines for A/D Converter setup
#define A_CLK      ((XTAL/1000000)-1)       // about 1MHz Analog Clock
#define ANA_CLK    (XTAL/(A_CLK+1))         // precise Analog Clock
#define DECIMATION (ANA_CLK/64/10)          // 10 Hz Decimation
#define CONV_FREQ  (ANA_CLK/64/DECIMATION)  // Conversion Frequency

#define RFGCLK						BIT7
#define ENABLE_BOD					BIT6
#define INT_VRREF					BIT5
#define EXT_VREF					0	
#define VREF1_25V					0
#define VREF2_5V					BIT4
#define ENABLE_BUFFER				BIT3
#define PGA_1						0
#define PGA_2						1
#define PGA_4						2
#define PGA_8						3
#define PGA_16						4
#define PGA_32						5
#define PGA_64						6
#define PGA_128						7

#define UNIPOLAR					BIT6
#define BIPOLAR						0


void ADC_Initialize(void)
{
	// Setup ADC
	ADMUX  = 0x08;                    // (AIN+ = AIN0), (AIN- = AINCOM) Voltage from DAC
	ACLK   = A_CLK;                   // set ACLK factor for about 1MHz

    // Vref On, Vref Hi, Buff off, BOD off, PGA
	//ADCON0 = 0x10;//INT_VRREF|VREF2_5V|PGA_1;
	ADCON0 = EXT_VREF|VREF2_5V|PGA_1;
	
	ADCON2 = DECIMATION & 0xFF;       // LSB of decimation
	ADCON3 = (DECIMATION>>8) & 0x07;   // MSB of decimation
	ADCON1 = 0x01|UNIPOLAR;           // unipolar, auto, self calibration (offset, gain)
}

// Helper structure to read in ADC values
typedef union
{
  unsigned char  c[4];              // bytes
  unsigned long  l;                 // unsigned long
}TYPE_ADC24;


unsigned int AdcRead24(unsigned char Channel, TYPE_ADC24 *pAdc24Data)
{
	unsigned char i = 0;
	ADMUX = (Channel<<4) | 8;        // AINP = chan, AINN = AINCOM
	
	// Read Input Volatage
	// ignore 4 conversions for filter to settle after calibration
	for (i=0; i<5; i++)
	{
		while (!(AIE & 0x20));      // Wait for data ready
	
		pAdc24Data->c[0] = ADRESH ^ 0x80;   // Invert the Most significant bit (only positiv inputs)
		pAdc24Data->c[1] = ADRESM;
		pAdc24Data->c[2] = ADRESL;
		pAdc24Data->c[3] = 0;
	}	
}

unsigned long AdcRead(unsigned char Channel)
{
	TYPE_ADC24 adc_buf;
	
	AdcRead24(Channel, &adc_buf);
	
	return adc_buf.l;
}
